[tor-commits] [tor/master] Throw control port warning if we failed to connect to all our bridges.

nickm at torproject.org nickm at torproject.org
Tue Mar 11 15:09:45 UTC 2014


commit 1c475eb018989f090c1423c25dc2f09380b10693
Author: George Kadianakis <desnacked at riseup.net>
Date:   Mon Mar 10 22:52:07 2014 +0000

    Throw control port warning if we failed to connect to all our bridges.
---
 changes/bug11069          |    4 ++++
 src/or/connection.c       |   25 +++++++++++++++++++++++++
 src/or/connection.h       |    2 ++
 src/or/connection_or.c    |    8 +++++---
 src/or/control.c          |   13 ++++++++-----
 src/or/control.h          |    3 ++-
 src/or/entrynodes.c       |   21 ---------------------
 src/or/entrynodes.h       |    1 -
 src/test/test_extorport.c |    4 +++-
 9 files changed, 49 insertions(+), 32 deletions(-)

diff --git a/changes/bug11069 b/changes/bug11069
new file mode 100644
index 0000000..5aa3085
--- /dev/null
+++ b/changes/bug11069
@@ -0,0 +1,4 @@
+  o Minor bugfixes (clients):
+    - Fix tor so that it raises a control port warning when we fail to
+      connect to all of our bridges. Fixes bug 11069; bugfix on
+      tor-0.2.1.2-alpha.
diff --git a/src/or/connection.c b/src/or/connection.c
index 77565ee..653fa1c 100644
--- a/src/or/connection.c
+++ b/src/or/connection.c
@@ -4164,6 +4164,31 @@ connection_dir_get_by_purpose_and_resource(int purpose,
   return NULL;
 }
 
+/** Return 1 if there are any active OR connections apart from
+ * <b>this_conn</b>.
+ *
+ * We use this to guess if we should tell the controller that we
+ * didn't manage to connect to any of our bridges. */
+int
+any_other_active_or_conns(const or_connection_t *this_conn)
+{
+  smartlist_t *conns = get_connection_array();
+  SMARTLIST_FOREACH_BEGIN(conns, connection_t *, conn) {
+    if (conn == TO_CONN(this_conn)) { /* don't consider this conn */
+      continue;
+    }
+
+    if (conn->type == CONN_TYPE_OR &&
+        !conn->marked_for_close) {
+      log_debug(LD_DIR, "%s: Found an OR connection: %s",
+                __func__, conn->address);
+      return 1;
+    }
+  } SMARTLIST_FOREACH_END(conn);
+
+  return 0;
+}
+
 /** Return 1 if <b>conn</b> is a listener conn, else return 0. */
 int
 connection_is_listener(connection_t *conn)
diff --git a/src/or/connection.h b/src/or/connection.h
index fa07650..13dcbcd 100644
--- a/src/or/connection.h
+++ b/src/or/connection.h
@@ -187,6 +187,8 @@ connection_t *connection_get_by_type_state_rendquery(int type, int state,
 dir_connection_t *connection_dir_get_by_purpose_and_resource(
                                            int state, const char *resource);
 
+int any_other_active_or_conns(const or_connection_t *this_conn);
+
 #define connection_speaks_cells(conn) ((conn)->type == CONN_TYPE_OR)
 int connection_is_listener(connection_t *conn);
 int connection_state_is_open(connection_t *conn);
diff --git a/src/or/connection_or.c b/src/or/connection_or.c
index dbf05a6..82b2971 100644
--- a/src/or/connection_or.c
+++ b/src/or/connection_or.c
@@ -714,7 +714,8 @@ connection_or_about_to_close(or_connection_t *or_conn)
                                      reason);
         if (!authdir_mode_tests_reachability(options))
           control_event_bootstrap_problem(
-                orconn_end_reason_to_control_string(reason), reason);
+                orconn_end_reason_to_control_string(reason),
+                reason, or_conn);
       }
     }
   } else if (conn->hold_open_until_flushed) {
@@ -1077,7 +1078,7 @@ connection_or_connect_failed(or_connection_t *conn,
 {
   control_event_or_conn_status(conn, OR_CONN_EVENT_FAILED, reason);
   if (!authdir_mode_tests_reachability(get_options()))
-    control_event_bootstrap_problem(msg, reason);
+    control_event_bootstrap_problem(msg, reason, conn);
 }
 
 /** <b>conn</b> got an error in connection_handle_read_impl() or
@@ -1708,7 +1709,8 @@ connection_or_client_learned_peer_id(or_connection_t *conn,
     if (!authdir_mode_tests_reachability(options))
       control_event_bootstrap_problem(
                                 "Unexpected identity in router certificate",
-                                END_OR_CONN_REASON_OR_IDENTITY);
+                                END_OR_CONN_REASON_OR_IDENTITY,
+                                conn);
     return -1;
   }
   if (authdir_mode_tests_reachability(options)) {
diff --git a/src/or/control.h b/src/or/control.h
index 0466de1..ce605a1 100644
--- a/src/or/control.h
+++ b/src/or/control.h
@@ -93,7 +93,8 @@ void monitor_owning_controller_process(const char *process_spec);
 
 void control_event_bootstrap(bootstrap_status_t status, int progress);
 MOCK_DECL(void, control_event_bootstrap_problem,(const char *warn,
-                                                 int reason));
+                                                 int reason,
+                                              const or_connection_t *or_conn));
 
 void control_event_clients_seen(const char *controller_str);
 void control_event_transport_launched(const char *mode,
diff --git a/src/or/entrynodes.c b/src/or/entrynodes.c
index d463303..17c5b13 100644
--- a/src/or/entrynodes.c
+++ b/src/or/entrynodes.c
@@ -2213,27 +2213,6 @@ any_bridge_descriptors_known(void)
   return choose_random_entry(NULL) != NULL;
 }
 
-/** Return 1 if there are any directory conns fetching bridge descriptors
- * that aren't marked for close. We use this to guess if we should tell
- * the controller that we have a problem. */
-int
-any_pending_bridge_descriptor_fetches(void)
-{
-  smartlist_t *conns = get_connection_array();
-  SMARTLIST_FOREACH_BEGIN(conns, connection_t *, conn) {
-    if (conn->type == CONN_TYPE_DIR &&
-        conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC &&
-        TO_DIR_CONN(conn)->router_purpose == ROUTER_PURPOSE_BRIDGE &&
-        !conn->marked_for_close &&
-        conn->linked &&
-        conn->linked_conn && !conn->linked_conn->marked_for_close) {
-      log_debug(LD_DIR, "found one: %s", conn->address);
-      return 1;
-    }
-  } SMARTLIST_FOREACH_END(conn);
-  return 0;
-}
-
 /** Return 1 if we have at least one descriptor for an entry guard
  * (bridge or member of EntryNodes) and all descriptors we know are
  * down. Else return 0. If <b>act</b> is 1, then mark the down guards
diff --git a/src/or/entrynodes.h b/src/or/entrynodes.h
index 772c666..73ac017 100644
--- a/src/or/entrynodes.h
+++ b/src/or/entrynodes.h
@@ -105,7 +105,6 @@ void retry_bridge_descriptor_fetch_directly(const char *digest);
 void fetch_bridge_descriptors(const or_options_t *options, time_t now);
 void learned_bridge_descriptor(routerinfo_t *ri, int from_cache);
 int any_bridge_descriptors_known(void);
-int any_pending_bridge_descriptor_fetches(void);
 int entries_known_but_down(const or_options_t *options);
 void entries_retry_all(const or_options_t *options);
 
diff --git a/src/test/test_extorport.c b/src/test/test_extorport.c
index 7e38ba5..b34f5e3 100644
--- a/src/test/test_extorport.c
+++ b/src/test/test_extorport.c
@@ -363,10 +363,12 @@ test_ext_or_cookie_auth_testvec(void *arg)
 }
 
 static void
-ignore_bootstrap_problem(const char *warn, int reason)
+ignore_bootstrap_problem(const char *warn, int reason,
+                       const or_connection_t *conn)
 {
   (void)warn;
   (void)reason;
+  (void)conn;
 }
 
 static int is_reading = 1;





More information about the tor-commits mailing list
diff --git a/src/or/control.c b/src/or/control.c
index 05ff9a6..48f9b57 100644
--- a/src/or/control.c
+++ b/src/or/control.c
@@ -4873,10 +4873,12 @@ control_event_bootstrap(bootstrap_status_t status, int progress)
 
 /** Called when Tor has failed to make bootstrapping progress in a way
  * that indicates a problem. <b>warn</b> gives a hint as to why, and
  * <b>reason</b> provides an "or_conn_end_reason" tag.
  */
 MOCK_IMPL(void,
-control_event_bootstrap_problem, (const char *warn, int reason))
+          control_event_bootstrap_problem, (const char *warn, int reason,
+                                            const or_connection_t *or_conn))
 {
   int status = bootstrap_percent;
   const char *tag, *summary;
@@ -4898,9 +4900,10 @@ control_event_bootstrap_problem, (const char *warn, int reason))
   if (reason == END_OR_CONN_REASON_NO_ROUTE)
     recommendation = "warn";
 
-  if (get_options()->UseBridges &&
-      !any_bridge_descriptors_known() &&
-      !any_pending_bridge_descriptor_fetches())
+  /* If we are using bridges and all our OR connections are now
+     closed, it means that we totally failed to connect to our
+     bridges. Throw a warning. */
+  if (get_options()->UseBridges && !any_other_active_or_conns(or_conn))
     recommendation = "warn";
 
   if (we_are_hibernating())
@@ -4873,10 +4873,12 @@ control_event_bootstrap(bootstrap_status_t status, int progress)
 /** Called when Tor has failed to make bootstrapping progress in a way
  * that indicates a problem. <b>warn</b> gives a hint as to why, and
- * <b>reason</b> provides an "or_conn_end_reason" tag.
+ * <b>reason</b> provides an "or_conn_end_reason" tag.  <b>or_conn</b>
+ * is the connection that caused this problem.
  */
 MOCK_IMPL(void,